@@ -23,12 +23,9 @@ import java.io.OutputStream;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.Collection;
-import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.StringTokenizer;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.logging.Level;
 import java.util.logging.Logger;
 
 import javax.wsdl.Definition;
@@ -48,7 +45,6 @@ import javax.xml.stream.XMLStreamWriter;
 
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
-import org.w3c.dom.NodeList;
 
 import org.xml.sax.InputSource;
 
@@ -57,6 +53,7 @@ import org.apache.cxf.catalog.OASISCatalogManager;
 import org.apache.cxf.common.i18n.Message;
 import org.apache.cxf.common.logging.LogUtils;
 import org.apache.cxf.helpers.CastUtils;
+import org.apache.cxf.helpers.DOMUtils;
 import org.apache.cxf.helpers.XMLUtils;
 import org.apache.cxf.service.model.EndpointInfo;
 import org.apache.cxf.staxutils.StaxUtils;
@@ -69,11 +66,13 @@ import org.apache.cxf.wsdl11.ServiceWSDLBuilder;
 public class WSDLQueryHandler implements StemMatchingQueryHandler {
     private static final Logger LOG = LogUtils.getL7dLogger(WSDLQueryHandler.class, "QueryMessages");
     private Bus bus;
-
+    
+    public WSDLQueryHandler() {
+    }
     public WSDLQueryHandler(Bus b) {
         bus = b;
     }
-
+    
     public String getResponseContentType(String baseUri, String ctx) {
         if (baseUri.toLowerCase().contains("?wsdl")
             || baseUri.toLowerCase().contains("?xsd=")) {
@@ -90,14 +89,15 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                 || baseUri.toLowerCase().contains("xsd=")))) {
             
             int idx = baseUri.indexOf("?");
-            Map<String, String> map = parseQueryString(baseUri.substring(idx + 1));
+            Map<String, String> map = UrlUtilities.parseQueryString(baseUri.substring(idx + 1));
             if (map.containsKey("wsdl")
                 || map.containsKey("xsd")) {
                 if (contextMatchExact) {
                     return endpointInfo.getAddress().contains(ctx);
                 } else {
                     // contextMatchStrategy will be "stem"
-                    return endpointInfo.getAddress().contains(getStem(baseUri.substring(0, idx)));
+                    return endpointInfo.getAddress().
+                                contains(UrlUtilities.getStem(baseUri.substring(0, idx)));
                 }
             }
         }
@@ -108,7 +108,7 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                               EndpointInfo endpointInfo, OutputStream os) {
         try {
             int idx = baseUri.toLowerCase().indexOf("?");
-            Map<String, String> params = parseQueryString(baseUri.substring(idx + 1));
+            Map<String, String> params = UrlUtilities.parseQueryString(baseUri.substring(idx + 1));
             String base = baseUri.substring(0, baseUri.toLowerCase().indexOf("?"));
             String wsdl = params.get("wsdl");
             String xsd =  params.get("xsd");
@@ -148,7 +148,12 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                     String wsdl2 = resolveWithCatalogs(OASISCatalogManager.getCatalogManager(bus),
                                                        wsdl,
                                                        base);
-                    def = mp.get(wsdl2);
+                    if (wsdl2 != null) {
+                        def = mp.get(wsdl2);
+                    }
+                }
+                if (def == null) {
+                    throw new WSDLQueryException(new Message("WSDL_NOT_FOUND", LOG, wsdl), null);
                 }
                 
                 synchronized (def) {
@@ -171,7 +176,12 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                     String xsd2 = resolveWithCatalogs(OASISCatalogManager.getCatalogManager(bus),
                                                        xsd,
                                                        base);
-                    si = smp.get(xsd2);
+                    if (xsd2 != null) { 
+                        si = smp.get(xsd2);
+                    }
+                }
+                if (si == null) {
+                    throw new WSDLQueryException(new Message("SCHEMA_NOT_FOUND", LOG, wsdl), null);
                 }
                 
                 String uri = si.getReferencedSchema().getDocumentBaseURI();
@@ -188,37 +198,7 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                 doc = XMLUtils.getParser().parse(src);
             }
             
-            NodeList nl = doc.getDocumentElement()
-                .getElementsByTagNameNS("http://www.w3.org/2001/XMLSchema",
-                                        "import");
-            for (int x = 0; x < nl.getLength(); x++) {
-                Element el = (Element)nl.item(x);
-                String sl = el.getAttribute("schemaLocation");
-                if (smp.containsKey(sl)) {
-                    el.setAttribute("schemaLocation", base + "?xsd=" + sl);
-                }
-            }
-            nl = doc.getDocumentElement()
-                .getElementsByTagNameNS("http://www.w3.org/2001/XMLSchema",
-                                        "include");
-            for (int x = 0; x < nl.getLength(); x++) {
-                Element el = (Element)nl.item(x);
-                String sl = el.getAttribute("schemaLocation");
-                if (smp.containsKey(sl)) {
-                    el.setAttribute("schemaLocation", base + "?xsd=" + sl);
-                }
-            }
-            nl = doc.getDocumentElement()
-                .getElementsByTagNameNS("http://schemas.xmlsoap.org/wsdl/",
-                                    "import");
-            for (int x = 0; x < nl.getLength(); x++) {
-                Element el = (Element)nl.item(x);
-                String sl = el.getAttribute("location");
-                if (mp.containsKey(sl)) {
-                    el.setAttribute("location", base + "?wsdl=" + sl);
-                }
-            }
-            doc.setXmlStandalone(true);
+            updateDoc(doc, base, mp, smp);
             String enc = doc.getXmlEncoding();
             if (enc == null) {
                 enc = "utf-8";
@@ -228,12 +208,49 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                                                                      enc);
             StaxUtils.writeNode(doc, writer, true);
             writer.flush();
+        } catch (WSDLQueryException wex) {
+            throw wex;
         } catch (Exception wex) {
             throw new WSDLQueryException(new Message("COULD_NOT_PROVIDE_WSDL",
                                                      LOG,
                                                      baseUri), wex);
         }
     }
+    
+    private void updateDoc(Document doc, String base,
+                           Map<String, Definition> mp,
+                           Map<String, SchemaReference> smp) {        
+        List<Element> elementList = DOMUtils.findAllElementsByTagNameNS(doc.getDocumentElement(),
+                                                                       "http://www.w3.org/2001/XMLSchema",
+                                                                       "import");
+        for (Element el : elementList) {
+            String sl = el.getAttribute("schemaLocation");
+            if (smp.containsKey(sl)) {
+                el.setAttribute("schemaLocation", base + "?xsd=" + sl);
+            }
+        }
+        
+        elementList = DOMUtils.findAllElementsByTagNameNS(doc.getDocumentElement(),
+                                                          "http://www.w3.org/2001/XMLSchema",
+                                                          "include");
+        for (Element el : elementList) {
+            String sl = el.getAttribute("schemaLocation");
+            if (smp.containsKey(sl)) {
+                el.setAttribute("schemaLocation", base + "?xsd=" + sl);
+            }
+        }
+        
+        elementList = DOMUtils.findAllElementsByTagNameNS(doc.getDocumentElement(),
+                                                          "http://schemas.xmlsoap.org/wsdl/",
+                                                          "import");
+        for (Element el : elementList) {
+            String sl = el.getAttribute("location");
+            if (mp.containsKey(sl)) {
+                el.setAttribute("location", base + "?wsdl=" + sl);
+            }
+        }
+        doc.setXmlStandalone(true);
+    }
 
     static String resolveWithCatalogs(OASISCatalogManager catalogs, String start, String base) {
         String resolvedSchemaLocation = null;
@@ -265,16 +282,18 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                 
                 if (resolvedSchemaLocation == null) {
                     try {
-                        //check to see if it's aleady in a URL format.  If so, leave it.
+                        //check to see if it's already in a URL format.  If so, leave it.
                         new URL(start);
                     } catch (MalformedURLException e) {
-                        done.put(start, imp.getDefinition());
-                        updateDefinition(imp.getDefinition(), done, doneSchemas, base, ei);
+                        if (done.put(start, imp.getDefinition()) == null) {
+                            updateDefinition(imp.getDefinition(), done, doneSchemas, base, ei);
+                        }
                     }
                 } else {
-                    done.put(start, imp.getDefinition());
-                    done.put(resolvedSchemaLocation, imp.getDefinition());
-                    updateDefinition(imp.getDefinition(), done, doneSchemas, base, ei);
+                    if (done.put(start, imp.getDefinition()) == null) {
+                        done.put(resolvedSchemaLocation, imp.getDefinition());
+                        updateDefinition(imp.getDefinition(), done, doneSchemas, base, ei);
+                    }
                 }
             }
         }      
@@ -340,16 +359,18 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                     String resolvedSchemaLocation = resolveWithCatalogs(catalogs, start, base);
                     if (resolvedSchemaLocation == null) {
                         try {
-                            //check to see if it's aleady in a URL format.  If so, leave it.
+                            //check to see if it's already in a URL format.  If so, leave it.
                             new URL(start);
                         } catch (MalformedURLException e) {
-                            doneSchemas.put(start, imp);
-                            updateSchemaImports(imp.getReferencedSchema(), doneSchemas, base);
+                            if (doneSchemas.put(start, imp) == null) {
+                                updateSchemaImports(imp.getReferencedSchema(), doneSchemas, base);
+                            }
                         }
                     } else {
-                        doneSchemas.put(start, imp);
-                        doneSchemas.put(resolvedSchemaLocation, imp);
-                        updateSchemaImports(imp.getReferencedSchema(), doneSchemas, base);
+                        if (doneSchemas.put(start, imp) == null) {
+                            doneSchemas.put(resolvedSchemaLocation, imp);
+                            updateSchemaImports(imp.getReferencedSchema(), doneSchemas, base);
+                        }
                     }
                 }
             }
@@ -366,8 +387,9 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
                             //check to see if it's aleady in a URL format.  If so, leave it.
                             new URL(start);
                         } catch (MalformedURLException e) {
-                            doneSchemas.put(start, included);
-                            updateSchemaImports(included.getReferencedSchema(), doneSchemas, base);
+                            if (doneSchemas.put(start, included) == null) {
+                                updateSchemaImports(included.getReferencedSchema(), doneSchemas, base);
+                            }
                         }
                     }
                 } else if (!doneSchemas.containsKey(start) 
@@ -383,40 +405,8 @@ public class WSDLQueryHandler implements StemMatchingQueryHandler {
     public boolean isRecognizedQuery(String baseUri, String ctx, EndpointInfo endpointInfo) {
         return isRecognizedQuery(baseUri, ctx, endpointInfo, false);
     }
-    
-      
-    private String getStem(String baseURI) {
-        
-        URL url = null;
-        try {
-            url = new URL(baseURI);
-        } catch (MalformedURLException e) {
-            LOG.log(Level.WARNING, "URL creation failed: ", e);
-        }
-        if (url != null) {
-            baseURI = url.getPath();
-            int idx = baseURI.lastIndexOf('/');
-            if (idx != -1) {
-                baseURI = baseURI.substring(0, idx);
-            }
-        }        
-        return baseURI;
+
+    public void setBus(Bus bus) {
+        this.bus = bus;
     }
-    
-    static Map<String, String> parseQueryString(String s) {
-        Map<String, String> ht = new HashMap<String, String>();
-        StringTokenizer st = new StringTokenizer(s, "&");
-        while (st.hasMoreTokens()) {
-            String pair = (String)st.nextToken();
-            int pos = pair.indexOf('=');
-            if (pos == -1) {
-                ht.put(pair.toLowerCase(), "");
-            } else {
-                ht.put(pair.substring(0, pos).toLowerCase(),
-                       pair.substring(pos + 1));
-            }
-        }
-        return ht;
-    }
-     
 }
